@@ -10,8 +10,13 @@ define([
     , './api_list.js'
     , './constData.js'
     , './block.js'
+
+    , './component/blockMenu.js'
+
+    , 'codemirror/lib/codemirror'
 ], function ( vpCommon, vpFuncJS, md, vpConst, sb,
-              shadowBlock, api, api_list, constData, block ) {
+              shadowBlock, api, api_list, constData, block, BlockMenu
+              , codemirror) {
 
     const { RemoveSomeBlockAndGetBlockList
         
@@ -184,6 +189,30 @@ define([
         this.mdHandler = null;  
 
         this.domPool = new Map();
+
+        var cmReadonlyConfig = {
+            mode: {
+                name: 'python',
+                version: 3,
+                singleLineStringErrors: false
+            },  // text-cell(markdown cell) set to 'htmlmixed'
+            height: '100%',
+            width: '100%',
+            indentUnit: 4,
+            matchBrackets: true,
+            readOnly:true,
+            autoRefresh: true,
+            // lineWrapping: false, // text-cell(markdown cell) set to true
+            // indentWithTabs: true,
+            theme: "ipython",
+            extraKeys: {"Enter": "newlineAndIndentContinueMarkdownList"},
+            scrollbarStyle: "null"
+        };
+        this.cmOverallPreview = codemirror.fromTextArea($('#vp_overallCodePreview')[0], cmReadonlyConfig);
+        this.cmOptionPreview = codemirror.fromTextArea($('#vp_optionCodePreview')[0], cmReadonlyConfig);
+
+        // blockmenu
+        this.blockMenu = new BlockMenu(this);
     }
 
     BlockContainer.prototype.setMetahandler = function(funcID) {
@@ -391,6 +420,8 @@ define([
 
     BlockContainer.prototype.setSelectBlock = function(selectedBlock) {
         this.selectedBlock = selectedBlock;
+
+        this.setOptionPreviewBox(selectedBlock);
     }
     BlockContainer.prototype.getSelectBlock = function() {
         return this.selectedBlock;
@@ -424,9 +455,9 @@ define([
     BlockContainer.prototype.setFocusedPageType = function(focusedPageType) {
         // 추가: FOCUSED_PAGE_TYPE = NULL일 경우 주피터 단축키 활성화 / 아닐 경우 비활성화
         if (focusedPageType == FOCUSED_PAGE_TYPE.NULL) {
-            Jupyter.notebook.keyboard_manager.enable();
+            // Jupyter.notebook.keyboard_manager.enable();
         } else {
-            Jupyter.notebook.keyboard_manager.disable();
+            // Jupyter.notebook.keyboard_manager.disable();
         }
         this.focusedPageType = focusedPageType;
     }
@@ -871,7 +902,9 @@ define([
 
                     this.resetOptionPage();
                 });
-            } 
+            } else if (blockType == BLOCK_CODELINE_TYPE.API) {
+                block.renderApiOption();
+            }
         });
 
         this.resetOptionPage();
@@ -1049,10 +1082,11 @@ define([
         //         lastBlockList_blockArea.push(lastBlock);
         //     });
         // }
+        
+        var lastBlockList_blockArea = []
         if (isBlock == true) {
             /** 각 node, text 블럭 영역의 가장 아래의 블럭들을 가져옴 */
             var nodeTextBlockList = blockContainerThis.getNodeBlockAndTextBlockList_asc();
-            var lastBlockList_blockArea = []
             nodeTextBlockList.forEach(nodeTextBlock => {
                 if (thisBlock.getUUID() == nodeTextBlock.getUUID()) {
                     return;
@@ -1071,7 +1105,6 @@ define([
             // if (blockType == BLOCK_CODELINE_TYPE.TEXT) {
             //     return;
             // }
-            
 
             if (isBlock == true) {
                 /**
@@ -1379,11 +1412,13 @@ define([
         /** def block일 경우 */
         } else if ( blockType == BLOCK_CODELINE_TYPE.DEF) {
 
+            var codeBlock = this.createBlock( BLOCK_CODELINE_TYPE.CODE );
             var returnBlock = this.createBlock( BLOCK_CODELINE_TYPE.RETURN );
             var holderBlock = this.createBlock( BLOCK_CODELINE_TYPE.SHADOW );
             
             thisBlock.appendBlock(holderBlock, BLOCK_DIRECTION.DOWN);
             thisBlock.appendBlock(returnBlock, BLOCK_DIRECTION.INDENT);
+            thisBlock.appendBlock(codeBlock, BLOCK_DIRECTION.INDENT);
 
             $(holderBlock.getBlockMainDom()).addClass(VP_BLOCK_BLOCKCODELINETYPE_CLASS_DEF);
             $(holderBlock.getBlockMainDom()).css(STR_BACKGROUND_COLOR, COLOR_CLASS_DEF);
@@ -1739,9 +1774,10 @@ define([
             sbMainHeader.appendFormatLine("{0}", classOrDefName);
             sbMainHeader.appendLine("</div>");
 
-            sbMainHeader.appendFormatLine("<div class='{0} {1}' style='{2}'>", `vp-block-header-param`, 
+            sbMainHeader.appendFormatLine("<div class='{0} {1}' style='{2}' title='{3}'>", `vp-block-header-param`, 
                                                                                 `vp-block-header-${blockUUID}`,
-                                                                                'text-indent: 0em');
+                                                                                'text-indent: 0em',
+                                                                                codeLineStr);
             sbMainHeader.appendFormatLine("{0}", codeLineStr);
             sbMainHeader.appendLine("</div>");
 
@@ -1800,7 +1836,7 @@ define([
                                                                                                                 ? 'background-color: transparent'
                                                                                                                 : '');
 
-            sbMainHeader.appendFormatLine("<div class='{0} {1}'>",'vp-block-header-param', `vp-block-header-${blockUUID}`);
+            sbMainHeader.appendFormatLine("<div class='{0} {1}' title='{2}'>",'vp-block-header-param', `vp-block-header-${blockUUID}`, codeLineStr);
             sbMainHeader.appendFormatLine("{0}", codeLineStr);
             sbMainHeader.appendLine("</div>");
 
@@ -1928,7 +1964,7 @@ define([
     }
 
     /** 코드 생성 */
-    BlockContainer.prototype.makeCode = function(thisBlock) {
+    BlockContainer.prototype.makeCode = function(thisBlock, asPreview = false) {
         var codeLine = STR_EMPTY;
         var rootDepth = thisBlock.getDepth();
         var blockList_thisBlockArea = thisBlock.getBlockList_thisBlockArea_noShadowBlock();
@@ -1948,7 +1984,11 @@ define([
                     codeList.push(apiCodeLine);
                 }
             } else {
-                codeList.push(block.setCodeLineAndGet(indentString, false));
+                var tmpCodeLine = block.setCodeLineAndGet(indentString, false)
+                if (asPreview && blockType == BLOCK_CODELINE_TYPE.TEXT && tmpCodeLine.length > 0) {
+                    tmpCodeLine = '# ' + tmpCodeLine.replaceAll('\n', '\n# ');
+                }
+                codeList.push(tmpCodeLine);
             }
         });
 
@@ -1962,7 +2002,7 @@ define([
     /** Run All 버튼 실행시 board에 있는 전체 코드 생성 
      * FIXME: 노드별로 셀 추가하도록 수정해야 함
     */
-    BlockContainer.prototype.makeAllCode = function() {
+    BlockContainer.prototype.makeAllCode = function(runcell = true, asPreview = false) {
         var rootBlock = this.getRootBlock();
         if (!rootBlock) {
             return;
@@ -2003,6 +2043,9 @@ define([
                 codeList.push( { type: 'code', data: [ codeLine ] } );
                 groupIdx = codeList.length - 1;
             } else if (blockType == BLOCK_CODELINE_TYPE.TEXT) {
+                if (asPreview && codeLine.length > 0) {
+                    codeLine = '# ' + codeLine.replaceAll('\n', '\n# ');
+                }
                 codeList.push( { type: 'markdown', data: [ codeLine ] } );
                 groupIdx = codeList.length - 1;
             } else {
@@ -2023,7 +2066,9 @@ define([
             }
             return { command: command, type: obj.type, exec: true }
         });
-        vpCommon.cellExecute(cmdList);
+        if (runcell) {
+            vpCommon.cellExecute(cmdList);
+        }
 
         this.setAPIBlockCode(codeLineStr);
         return codeLineStr;
@@ -2075,26 +2120,26 @@ define([
         var buttonsPageRectWidth = NUM_BUTTONS_PAGE_WIDTH; 
         var boardPageRect =  $(vpCommon.wrapSelector(VP_CLASS_PREFIX + VP_CLASS_APIBLOCK_BOARD_CONTAINER))[0].getBoundingClientRect(); 
         var boardPageRectWidth = boardPageRect.width;
-        var optionPageRectWidth = mainPageRectWidthNum - boardPageRectWidth - buttonsPageRectWidth - 50;
+        var optionPageRectWidth = mainPageRectWidthNum - boardPageRectWidth - buttonsPageRectWidth - 23;
 
         if (optionPageRectWidth < NUM_OPTION_PAGE_WIDTH) {
-            boardPageRectWidth = mainPageRectWidthNum - NUM_OPTION_PAGE_WIDTH - 50 - buttonsPageRectWidth;
-            // optionPageRectWidth = NUM_OPTION_PAGE_WIDTH;
+            boardPageRectWidth = mainPageRectWidthNum - NUM_OPTION_PAGE_WIDTH - 23 - buttonsPageRectWidth;
+            optionPageRectWidth = NUM_OPTION_PAGE_WIDTH;
         } 
         
-        var optionPageRectWidth_maxWidth = mainPageRectWidthNum - buttonsPageRectWidth - NUM_API_BOARD_CENTER_PAGE_MIN_WIDTH - 50;
+        var optionPageRectWidth_maxWidth = mainPageRectWidthNum - buttonsPageRectWidth - NUM_API_BOARD_CENTER_PAGE_MIN_WIDTH - 23;
 
         $(vpCommon.wrapSelector(VP_CLASS_PREFIX + VP_CLASS_APIBLOCK_OPTION_TAB)).css(STR_MAX_WIDTH, optionPageRectWidth_maxWidth);
         $(vpCommon.wrapSelector(VP_CLASS_PREFIX + VP_CLASS_APIBLOCK_BOARD_CONTAINER)).css(STR_WIDTH, boardPageRectWidth );
         $(vpCommon.wrapSelector(VP_CLASS_PREFIX + VP_CLASS_APIBLOCK_OPTION_TAB)).css(STR_WIDTH, optionPageRectWidth );
-        // $(VP_ID_PREFIX + VP_ID_WRAPPER).css(STR_MIN_WIDTH, 760);
+        $(VP_ID_PREFIX + VP_ID_WRAPPER).css(STR_MIN_WIDTH, 830);
 
-        this.setBlockMaxWidth_blockList(boardPageRectWidth - 70);
+        this.setBlockMaxWidth_blockList(boardPageRectWidth - 60);
 
         /** node 블럭 width 재 계산 */
         var nodeBlockList = this.getNodeBlockList();
         nodeBlockList.forEach(nodeBlock => {
-            nodeBlock.renderNodeBlockWidth(boardPageRectWidth - 70);
+            nodeBlock.renderNodeBlockWidth(boardPageRectWidth - 60);
         });
     }
 
@@ -2119,13 +2164,13 @@ define([
 
         $(vpCommon.wrapSelector(VP_CLASS_PREFIX + VP_CLASS_APIBLOCK_BOARD_CONTAINER)).css(STR_WIDTH, boardPageRectWidth );
         $(vpCommon.wrapSelector(VP_CLASS_PREFIX + VP_CLASS_APIBLOCK_OPTION_TAB)).css(STR_WIDTH, optionPageRectWidth);
-        
-        this.setBlockMaxWidth_blockList(boardPageRectWidth - 70);
+    
+        this.setBlockMaxWidth_blockList(boardPageRectWidth - 60);
         
        /** node 블럭 width 재 계산 */
         var nodeBlockList = this.getNodeBlockList();
         nodeBlockList.forEach(nodeBlock => {
-            nodeBlock.renderNodeBlockWidth(boardPageRectWidth - 70);
+            nodeBlock.renderNodeBlockWidth(boardPageRectWidth - 60);
         });
     }
     
@@ -2134,14 +2179,16 @@ define([
      *  max width를 토대로 블럭의 width를 다시 표시함
      */
     BlockContainer.prototype.setBlockMaxWidth_blockList = function(boardPageRectWidth) {
+        var blockMaxWidth = boardPageRectWidth;
+        this.setBlockMaxWidth(boardPageRectWidth);
         var blockList = this.getBlockList();
         blockList.forEach(block => {
-            var blockMaxWidth = boardPageRectWidth;
-            this.setBlockMaxWidth(blockMaxWidth);
-
+            
             var indentDepthNum = block.getIndentNumber();
             var blockWidth = blockMaxWidth - indentDepthNum;
             var blockMainDom = block.getBlockMainDom();
+
+            block.setWidth(blockWidth);
             $(blockMainDom).css(STR_WIDTH, blockWidth);
         });
     }
@@ -2244,6 +2291,26 @@ define([
 
     /** ---------------------option popup dom 관련 메소드 --------------------------------------- */
     //
+
+    BlockContainer.prototype.setNavigator = function(blockType, title) {
+        var tag = new sb.StringBuilder();
+        if (blockType == BLOCK_CODELINE_TYPE.API) {
+            // api blocks
+            tspl = title.split('>');
+            tspl.push(vpCommon.formatString('<strong>{0}</strong>', tspl.pop()));
+            tag.appendLine(tspl.join('>'));
+        } else if (blockType == BLOCK_CODELINE_TYPE.NONE) {
+            // visual python preview
+            tag.appendFormatLine('<span class="vp-orange-text">{0}</span>', title);
+        } else {
+            // blocks
+            tag.appendFormatLine('<strong>{0}</strong>', title);
+        }
+
+        // set navigator label
+        $('.vp-apiblock-option-navigator-label').html(tag.toString());
+    }
+
     BlockContainer.prototype.setOptionDom = function(UUID, type, blockOptionPageDom_new) {
         this.setOptionDomPool_none();
         if (this.domPool.get(UUID)) {
@@ -2311,6 +2378,9 @@ define([
             $(blockOptionPageDom_old).css(STR_DISPLAY, STR_NONE);
         }
 
+        $('.vp-apiblock-option-tab-none').css(STR_DISPLAY, STR_NONE);
+        $('.vp-apiblock-option-tab-preview').css(STR_DISPLAY, STR_NONE);
+
         var optionPageSelector = this.getOptionPageSelector();
         $(optionPageSelector + ' .vp-apiblock-option').css(STR_DISPLAY, STR_NONE);
         $(optionPageSelector + ' .vp-option-page').css(STR_DISPLAY, STR_NONE);
@@ -2320,10 +2390,67 @@ define([
     BlockContainer.prototype.resetOptionPage = function() {
         this.setOptionDomPool_none();
         this.setSelectBlock(null);
+        this.hideOptionPreviewBox();
 
+        this.setNavigator(BLOCK_CODELINE_TYPE.NONE, 'Visual Python 1.0.6');
+        this.setFocusedPageType(FOCUSED_PAGE_TYPE.BOARD);
         $('.vp-apiblock-option-tab-none').css(STR_DISPLAY, STR_BLOCK);
     }
 
+    BlockContainer.prototype.setOptionPageWithPreview = function(codeLineStr) {
+        this.setOptionDomPool_none();
+        this.setSelectBlock(null);
+
+        // codemirror with vp_overallCodePriview
+        this.cmOverallPreview.setValue(codeLineStr);
+        this.cmOverallPreview.save();
+        this.cmOverallPreview.focus();
+        var that = this;
+        // this.cmOverallPreview.setCursor({ line: codeLineStr.split('\n').length, ch: 1 });
+        setTimeout(function() {
+            that.cmOverallPreview.refresh();
+        },1);
+        $('.vp-apiblock-option-tab-preview').css(STR_DISPLAY, STR_BLOCK);
+    }
+
+    BlockContainer.prototype.showOptionPreviewBox = function() {
+        var selectedBlock = this.getSelectBlock();
+        this.setOptionPreviewBox(selectedBlock);
+        
+        if (selectedBlock == null) {
+            var rootBlock = this.getRootBlock();
+            if (rootBlock) {
+                var blockList = rootBlock.getRootToChildBlockList();
+                if (blockList && blockList.length > 0) {
+                    var allCodeLineStr = this.makeAllCode(false, true);
+                    this.setOptionPageWithPreview(allCodeLineStr);
+                    this.setNavigator(BLOCK_CODELINE_TYPE.NODE, 'Overall Code Preview');
+                    this.setFocusedPageType(FOCUSED_PAGE_TYPE.BOARD);
+                }
+            }
+            $('.vp-apiblock-option-preview-container').css(STR_DISPLAY, STR_NONE);
+            $(VP_ID_PREFIX + VP_APIBLOCK_BOARD_OPTION_PREVIEW_BUTTON).removeClass('enabled');
+        } else {
+            $('.vp-apiblock-option-preview-container').css(STR_DISPLAY, STR_BLOCK);
+        }
+    }
+
+    BlockContainer.prototype.setOptionPreviewBox = function(selectedBlock) {
+        var that = this;
+
+        var codeLineStr = selectedBlock == null? '': this.makeCode(selectedBlock, true);
+
+        this.cmOptionPreview.setValue(codeLineStr);
+        this.cmOptionPreview.save();
+        setTimeout(function() {
+            that.cmOptionPreview.refresh();
+        },1);
+    }
+
+    BlockContainer.prototype.hideOptionPreviewBox = function() {
+        $('.vp-apiblock-option-preview-container').css(STR_DISPLAY, STR_NONE);
+    }
+
     BlockContainer.prototype.getBoardPage_$ = function() {
         var $boardPage = $(vpCommon.wrapSelector(VP_CLASS_PREFIX + VP_CLASS_APIBLOCK_BOARD));
         return $boardPage;
