@@ -90,8 +90,7 @@ define([
     ImportPackage.prototype.initHtml = function() {
         var that = this;
         this.loadCss(Jupyter.notebook.base_url + vpConst.BASE_PATH + vpConst.STYLE_PATH + API_BLOCK_INDEX_CSS);
-        this.blockContainer = init();
-        this.blockContainer.setImportPackageThis(this);
+        this.blockContainer = init(this);
         this.blockContainer.setAPIBlockMetadataHandler();
 
         if (this.metadata) {
