@@ -41,6 +41,12 @@ define([
             , VP_ID_APIBLOCK_BOARD_MAKE_NODE_PATH
             , VP_ID_APIBLOCK_DELETE_BLOCK_ALL
 
+            , VP_ID_APIBLOCK_MENU_BOX
+
+            , VP_APIBLOCK_BOARD_OPTION_PREVIEW_BUTTON
+            , VP_APIBLOCK_BOARD_OPTION_CANCEL_BUTTON
+            , VP_APIBLOCK_BOARD_OPTION_APPLY_BUTTON
+
             , VP_CLASS_PREFIX
             , VP_CLASS_BLOCK_GROUPBOX_PREFIX
             , VP_CLASS_APIBLOCK_BOARD_CONTAINER
@@ -119,7 +125,14 @@ define([
         var createLogicGroupArray = Object.values(BLOCK_GROUP_TYPE);
         var logicBlockContainer = VP_CLASS_PREFIX + VP_CLASS_BLOCK_GROUPBOX_PREFIX + 'logic';
         createLogicGroupArray.forEach(enumData => {
-            new CreateGroup(blockContainer, enumData, MapGroupTypeToName(enumData), logicBlockContainer);
+            var groupName = MapGroupTypeToName(enumData);
+            var open = false;
+            var defaultOpenGroupList = ['Define', 'Control'];
+            // open defined group div
+            if (defaultOpenGroupList.includes(groupName)) {
+                open = true;
+            }
+            new CreateGroup(blockContainer, enumData, MapGroupTypeToName(enumData), logicBlockContainer, 0, open);
         });
 
         /** Logic(Define, Control, Execute)에 블럭 생성 버튼 생성 */
@@ -139,17 +152,24 @@ define([
 
         /** 추가: FIXME: Data Analysis 메뉴 임시 추가 */
         var TEMP_DA_MENUS = [
-            'Data Preprocessing',
-            'Data Exploration',
-            'Visualization',
-            'DataBase',
+            'Database',
             'Crawling',
-            'Machine Learning',
-            'Deep Learning'
+            'Data Preprocessing',
+            'EDA',
+            'Visualization',
+            'Text Analysis'
         ];
         TEMP_DA_MENUS.forEach((menu, idx) => {
             new CreateGroup(blockContainer, 'da_' + idx, menu, VP_CLASS_PREFIX + 'vp-block-group-box-da');
         });
+        /** 추가: FIXME: AI 메뉴 임시 추가 */
+        var TEMP_AI_MENUS = [
+            'Machine Learning',
+            'Deep Learning'
+        ]
+        TEMP_AI_MENUS.forEach((menu, idx) => {
+            new CreateGroup(blockContainer, 'ai_' + idx, menu, VP_CLASS_PREFIX + 'vp-block-group-box-ai');
+        });
 
         /** API Block 햄버거 메뉴바 생성 */
         apiBlockMenuInit(blockContainer);
@@ -165,7 +185,7 @@ define([
         /** 가운데 board 영역의 width */
         var boardPageRectWidth = NUM_API_BOARD_CENTER_PAGE_WIDTH;
         /** 오른쪽 option 영역의 width 계산 */
-        var optionPageRectWidth = mainPageRectWidthNum - buttonsPageRectWidth - boardPageRectWidth - 103;
+        var optionPageRectWidth = mainPageRectWidthNum - buttonsPageRectWidth - boardPageRectWidth - 73;
 
         /** visual python 전체의 width 렌더링 */
         $(vpCommon.wrapSelector(VP_ID_PREFIX + VP_ID_WRAPPER)).css(STR_WIDTH, mainPageRectWidth);
@@ -175,7 +195,8 @@ define([
         $(vpCommon.wrapSelector(VP_CLASS_PREFIX + VP_CLASS_APIBLOCK_OPTION_TAB)).css(STR_WIDTH, optionPageRectWidth);
         
         /** 오른쪽 option 영역의 max-width 렌더링*/
-        var optionPageRectWidth_maxWidth = mainPageRectWidthNum - buttonsPageRectWidth - 290 - 103;
+        // var optionPageRectWidth_maxWidth = mainPageRectWidthNum - buttonsPageRectWidth - 290 - 103;
+        var optionPageRectWidth_maxWidth = mainPageRectWidthNum - buttonsPageRectWidth - boardPageRectWidth - 8 - 73;
         $(vpCommon.wrapSelector(VP_CLASS_PREFIX + VP_CLASS_APIBLOCK_OPTION_TAB)).css(STR_MAX_WIDTH, optionPageRectWidth_maxWidth);
 
         /** 가운데 board 위 영역의 렌더링 */
@@ -192,35 +213,35 @@ define([
         blockContainer.reNewContainerDom();
 
         /** Block Max Width 설정 */
-        blockContainer.setBlockMaxWidth(boardPageRectWidth - 70);
+        blockContainer.setBlockMaxWidth(boardPageRectWidth - 60);
         
         /** delete 키 or 엔터 키를 누를 때 keyup 이벤트 발동
          */
-        $(document).keyup(function(e) {
-            var keycode =  e.keyCode 
-                                ? e.keyCode 
-                                : e.which;
+        // $(document).keyup(function(e) {
+        //     var keycode =  e.keyCode 
+        //                         ? e.keyCode 
+        //                         : e.which;
 
-            var selectedBlock = blockContainer.getSelectBlock();
+        //     var selectedBlock = blockContainer.getSelectBlock();
 
-            /** Delete 이벤트 */
-            /** block을 클릭하고 delete 키 눌렀을 때 */ 
-            if (keycode == NUM_DELETE_KEY_EVENT_NUMBER
-                && selectedBlock){
-                selectedBlock.deleteBlock_childBlockList();
-                blockContainer.resetOptionPage();
-                blockContainer.reRenderAllBlock_asc();
-            } 
+        //     /** Delete 이벤트 */
+        //     /** block을 클릭하고 delete 키 눌렀을 때 */ 
+        //     if (keycode == NUM_DELETE_KEY_EVENT_NUMBER
+        //         && selectedBlock){
+        //         selectedBlock.deleteBlock_childBlockList();
+        //         blockContainer.resetOptionPage();
+        //         blockContainer.reRenderAllBlock_asc();
+        //     } 
 
-            /** 엔터 이벤트 */
-            /** node 블럭을 클릭하고 엔터키를 눌렀을 때 */
-            if (keycode == NUM_ENTER_KEY_EVENT_NUMBER
-                && selectedBlock
-                && selectedBlock.getBlockType() == BLOCK_CODELINE_TYPE.NODE){
-                // console.log('엔터');
-                selectedBlock.renderNodeBlockInput(STR_NONE);
-            }
-        });
+        //     /** 엔터 이벤트 */
+        //     /** node 블럭을 클릭하고 엔터키를 눌렀을 때 */
+        //     if (keycode == NUM_ENTER_KEY_EVENT_NUMBER
+        //         && selectedBlock
+        //         && selectedBlock.getBlockType() == BLOCK_CODELINE_TYPE.NODE){
+        //         // console.log('엔터');
+        //         selectedBlock.renderNodeBlockInput(STR_NONE);
+        //     }
+        // });
 
         /** Logic, API, Data Analysis 의 > 버튼 클릭 */
         $(document).on(STR_CLICK,`.vp-apiblock-panel-area-vertical-btn`, function(){
@@ -290,7 +311,6 @@ define([
                     || $(event.target).attr('class').indexOf('vp-block-container') != -1 ) ) {
                 blockContainer.resetBlockList();
                 blockContainer.resetOptionPage();
-                blockContainer.setFocusedPageType(FOCUSED_PAGE_TYPE.EDITOR);
             }
         });
 
@@ -342,32 +362,33 @@ define([
                 }
             });
 
-            $(document).keydown(function(e) {
-                /** board에 선택한 블럭 없거나
-                *  TEXT 블럭인 경우에 ctrl + c , ctrl + v 금지
-                */
-                var selectedBlock = blockContainer.getSelectBlock();
-                if (!selectedBlock 
-                    || selectedBlock.getBlockType() == BLOCK_CODELINE_TYPE.TEXT
-                    || IsCodeBlockType(selectedBlock.getBlockType()) == true ) {
-                    return;
-                }   
-                // console.log('selectedBlock',selectedBlock);
+            // DEPRECATED: ctrl+c / ctrl+v 삭제
+            // $(document).keydown(function(e) {
+            //     /** board에 선택한 블럭 없거나
+            //     *  TEXT 블럭인 경우에 ctrl + c , ctrl + v 금지
+            //     */
+            //     var selectedBlock = blockContainer.getSelectBlock();
+            //     if (!selectedBlock 
+            //         || selectedBlock.getBlockType() == BLOCK_CODELINE_TYPE.TEXT
+            //         || IsCodeBlockType(selectedBlock.getBlockType()) == true ) {
+            //         return;
+            //     }   
+            //     // console.log('selectedBlock',selectedBlock);
 
-                /** ctrl + c */
-                if (ctrlDown 
-                    && e.ctrlKey && (e.keyCode == cKey)
-                    && !e.shiftKey && !e.altKey ) {
-                    blockContainer.setCtrlSaveData();
-                }
+            //     /** ctrl + c */
+            //     if (ctrlDown 
+            //         && e.ctrlKey && (e.keyCode == cKey)
+            //         && !e.shiftKey && !e.altKey ) {
+            //         blockContainer.setCtrlSaveData();
+            //     }
               
-                /** ctrl + v */
-                if (ctrlDown 
-                    && e.ctrlKey && (e.keyCode == vKey) 
-                    && !e.shiftKey && !e.altKey ) {
-                    blockContainer.copyCtrlSaveData();
-                }
-            });
+            //     /** ctrl + v */
+            //     if (ctrlDown 
+            //         && e.ctrlKey && (e.keyCode == vKey) 
+            //         && !e.shiftKey && !e.altKey ) {
+            //         blockContainer.copyCtrlSaveData();
+            //     }
+            // });
         });
 
         /** +node 블럭 생성 버튼 클릭 함수 바인딩 */
@@ -380,6 +401,29 @@ define([
             blockContainer.createTextBlock();
         });
 
+        /** option page - preview 버튼 클릭 함수 바인딩 */
+        $(document).on(STR_CLICK, VP_ID_PREFIX + VP_APIBLOCK_BOARD_OPTION_PREVIEW_BUTTON, function() {
+            if ($(this).hasClass('enabled')) {
+                blockContainer.hideOptionPreviewBox();
+                $(this).removeClass('enabled');
+            } else {
+                blockContainer.showOptionPreviewBox();
+                $(this).addClass('enabled');
+            }
+        });
+
+        /** option page - cancel 버튼 클릭 함수 바인딩 */
+        $(document).on(STR_CLICK, VP_ID_PREFIX + VP_APIBLOCK_BOARD_OPTION_CANCEL_BUTTON, function() {
+            blockContainer.resetBlockList();
+            blockContainer.resetOptionPage();
+        });
+
+        /** option page - apply 버튼 클릭 함수 바인딩 */
+        $(document).on(STR_CLICK, VP_ID_PREFIX + VP_APIBLOCK_BOARD_OPTION_APPLY_BUTTON, function() {
+            // TODO:
+            
+        });
+
         /**
          * API List item 클릭 이벤트 함수 바인딩
          * API List 목록의 함수에 대응되는 API List 블럭을 생성하기 위해서
@@ -469,7 +513,7 @@ define([
                     saveNotePageAction_newVersion(vpCommon.formatString("{0}.{1}", saveFileName, vpConst.VP_NOTE_EXTENSION), saveFilePath);
                     // apiBlockPackage.openMultiBtnModal_new('Save As', `Save changes to '${saveFileName}.vp'`,['Yes', 'No', 'Cancel'], [() => {
                     // },() => {
-        
+                        
                     // },() => {
                         
                     // }]);
@@ -512,6 +556,11 @@ define([
             }
         });
 
+        $(document).on('click', function(evt) {
+            // close block menu
+            $(vpCommon.wrapSelector(vpCommon.formatString('#{0}', VP_ID_APIBLOCK_MENU_BOX))).hide();
+        });
+
         ControlToggleInput();
         return blockContainer;
     }
